clear all;
load usedata;


% prepare data
%%%%%%%%%%%%%%

% observable vector for main VAR estimation
W = [einf abwf agch ccra sgr_filt' egch erp_4qf ey by]; 

% data definitions (see text for a detailed explanation)
% einf: survey-based expected inflation
% abwf: ang, Bekaert, Wei based real short rate
% agch: real cash flow growth
% ccra: campbell-cochrane based risk aversion measure
% sgr_filt: survey-based real uncertatinty
% egch: realized earnings growth
% erp_4qf: survey-based 4-qtr expected earnings growth
% ey: equity yield
% by: bond yield

% END prepare data
%%%%%%%%%%%%%%%%%%



% run cumby huizinga test
%%%%%%%%%%%%%%%%%%%%%%%%%

for ii = 1:size(W,2),
    % one lag VAR spec
     ytest = W(2:end,ii);
     ztest = [ones(size(W,1)-1,1) W(1:end-1,:)];
    % two lag VAR spec
     %ytest = W(3:end,ii);
     %ztest = [ones(size(W,1)-2,1) W(2:end-1,:) W(1:end-2,:)];
[b,bint,rtest,rint,stats] = regress(ytest,ztest);
ltest(ii,:) = cumby_huizinga_test(ztest,rtest,4,0);
end;

sprintf('cumby-huizinga results')
ltest


% END run cumby huizinga test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% define some parameters
%%%%%%%%%%%%%%%%%%%%%%%%

% dimension of VAR
k = size(W,2);
% dimension of endogenous variables
nZ = k+10;
% dimension of vector containing information variables, x_t
nInst = 4;
% horizon of survey-based earnings expectations
horiz = 4;
% selection vector for endogenous variael to be orthogonalized
% with respect to risk aversion and real uncertainty
e11 = (k+3==(1:nZ))';
% selection vector for risk aversion and real uncertainty
e21 = [(4==(1:nZ))' (5==(1:nZ))'];
% location of variation forecasted by survey data (earnings growth)
ftarg = 6;
% location of survey-based earnings forecast
sforc = 7;

% END define some parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% estimate observable VAR
rest0 = 0;
[muW,AW,CovW,PW,CovPW,sresid,rest1] = ecevar(W,1,rest0);
% AW is reported in Table 1
% but we use bootstrapped standrd errors, not the asymptotic ones
% produced by ecevar

% demean data, define some parameters
W = W - repmat(mean(W),size(W,1),1);
% length of sample
bigt = size(W,1);

% define mean of equity and bond yields
mey = mean(ey);
mby = mean(by);

% define Campbell-shiller (1988) linearization constants
% for this sample
rho = 1/(1+exp(mey));
kap = -log(rho)-(1-rho)*log(1/rho-1);

% define mean of data via VAR estimates
Wbar = inv(eye(size(AW))-AW)*muW;

% number of bootstrap draws
boot = 10000;

% every bootstrapped VAR sample
% will begin at the actual initial
% data point, W1
W1 = W(1,:);

%%%%%%%%%%%%%%%%%%%%%%%%%% %
% BEGIN BOOTSTRAPPING LOOP %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

jj = 0;
ii = 1;
while ii <= boot,
  
    if ii/100 == floor(ii/100),
        ii
        boot
    end;
    
% sample VAR errors with replacement
% to maintain a constant set of draws, fix 
% random number generator state
rand('state',123456+ii*10000+1000*jj);
c = floor(rand(size(sresid,1),1)*size(sresid,1))+1;
boot_sresid = sresid(c,:);


% simulate VAR based on bootstrapped 
% residuals
Wb = varsim(muW,AW,CovW,boot_sresid,W1);

% for the first 'draw,' ii==1, just use actual data
% (these are the point estimates)
if ii == 1,
    Wb = W;
end;

% get var coefs on bootstrapped sample
[muWb,AWb,CovWb] = ecevar(Wb,1,rest0);

% store transition matrix estimates
AWboot(:,:,ii) = AWb;


% invert Yt dynamics
%%%%%%%%%%%%%%%%%%%%%%%%

% initialize M matrices (see text for definition)
M1 = zeros(k);
M2 = zeros(k);
M3 = zeros(k);

% definte matriced for PV rotation

% number of variables that are simply copied
% from W to Y
ncopy = size(AW,1)-2;
M1(1:ncopy,1:ncopy) = eye(ncopy);

% one-period expected (real) equity return matrices is (end-1) element of Y
% (end-1) element of W is equity yield
M1(end-1,:) = [ 0   1  0  zeros(1,nInst)    1  0];
M2(end-1,:) = [ 0 rho -1  zeros(1,nInst)  rho  0];

% inflation risk premium (end) element of Y
% (end) element of W is nominal bond yield
M1(end,:)   = [ 1   1  0  zeros(1,nInst)    0  1];

% rotate observable VAR parameters to get latent VAR 
% parameteres
[AY, CovY, Theta0] = irotatevar(AWb,CovWb,M1,M2,M3,rho,40);
% calculate unconditional covariance matric of Yt
UCovY = reshape(inv(eye(k^2)-kron(AY,AY))*reshape(CovY,k^2,1),k,k);
% calculate unconditional volatility of Yt
UStdY = sqrt(diag(UCovY));

% store some Yt VAR-based statistics
AYb(:,:,ii) = AY;
StdYb(:,:,ii) = UStdY;
CorrYb(:,:,ii) = UCovY./ (UStdY*UStdY');
ACorrYb(:,:,ii) =  (AY*UCovY)./(UStdY*UStdY');

% END invert Yt dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%

% calculate some present values that are the 
% components of equity and bond yields

m0 = zeros(nZ,1);
M1 = zeros(nZ,k);
M2 = zeros(nZ,k);
M3 = zeros(nZ,k);

M1(1:k,1:k) = eye(k);

% equity yield components (a la Campbell-Shiller decomposition)
% k+1 = expected cash flow growth
% k+2 = expected return to real long bond
% k+3 = equity risk premium
% k+4 = total equity yield;

M1(k+2,:) = [ 0 rho  0  zeros(1,nInst)  0  0 ];
M1(k+3,:) = [ 0   0  0  zeros(1,nInst)  rho  0 ];
M2(k+1,:) = [ 0   0 -1  zeros(1,nInst)  0  0 ];
M2(k+2,:) = [ 0   1  0  zeros(1,nInst)  0  0 ];
M2(k+3,:) = [ 0   0  0  zeros(1,nInst)  1  0 ];
M1(k+4,:) = [ 0   0  0  zeros(1,nInst)  1  0 ]*inv(Theta0);

% bonds pv
% k+5 = expected inflation
% k+6 = expected return to real long bond
% k+7 = inflation risk premium
% k+8 = total bond yield;
M1(k+5,:) = [ 1  0  0  zeros(1,nInst) 0  0 ];
M1(k+6,:) = [ 0  1  0  zeros(1,nInst) 0  0 ];
M1(k+7,:) = [ 0  0  0  zeros(1,nInst) 0  1 ];
M1(k+8,:) = [ 0  0  0  zeros(1,nInst) 0  1 ]*inv(Theta0);

% calculate four-period period earnings growth VAR expectation,
% make into k+9 th element of the endogenous variable
% vector
M3(k+9,:) = ftarg==(1:k);

% now do all the expectations calculations
% for the endogenous variables
ThetaZ = rotatevar(AY,CovY,M1,M2,M3,rho,horiz);

% k+10 element will be the 'bias' measure:
% subjective forecast *minus* VAR forecast of four-quarter earnings growth
ThetaZ(k+10,:) = (sforc==(1:k)) - ThetaZ(k+9,:);

% ORTHOGONALIZATION of equity premium
% get new rotation with orthogonalized components
% equity risk premium orthogonalized to risk aversion
% and real uncertainty
% adds two new rows to ThetaZ, and 
% puts the spanned and orthogonal pieces in the sencod-to-
% last and last rows, respectively
[ThetaZ2,R2t] = varorthog(AY,CovY,ThetaZ,e11,e21);

% store R-squared of dependence of equity premium
% on risk factors
R2(:,:,ii) = R2t;

% decompose variation of all (10) the endogenous variables
% into compnents due to variation in elements of Yt
[uncvarZ uncAcorZ] = vardecomp(AY,CovY,ThetaZ2);

% selection matrix for ey and by components
% (this should be taken out ofthe loop at some point);
d1 = [k+1:k+8];

% select main equity compoents
vec1 = (k+1:k+3);
sum1 = k+4;
% select main bond components
vec2 = (k+5:k+7);
sum2 = k+8;

% calculate percentage
% variation in sum1 (sum2) due to
% individial variables in vec1 (vec2)
[p1,p2,p3] = varpctvar(uncvarZ,vec1,sum1,vec2,sum2);

% store portion of equity (bond) yield variance due to
% the three components, and the covariance between equity and bond yields
pe1b(:,:,ii) = sum(p1,2);
pb1b(:,:,ii) = sum(p2,2);
pc1b(:,:,ii) = p3;

% now redefine vec1 to split up
% the equity premium component into
% parts spanned-by and orthogonal-to
% the risk factors
sZ = size(uncvarZ,1);
vec1 = [(k+1:k+2) (sZ-1) (sZ-0)];
sum1 = k+4;
vec2 = (k+5:k+7);
sum2 = k+8;

[p1,p2,p3] = varpctvar(uncvarZ,vec1,sum1,vec2,sum2);

% store these decompositions
pe2b(:,:,ii) = sum(p1,2);
pb2b(:,:,ii) = sum(p2,2);
pc2b(:,:,ii) = p3;

% store some other reported stats from the 
% variance of equity yield
use(ii) = sqrt(uncvarZ(k+4,k+4));
% variance of bond yield
usb(ii) = sqrt(uncvarZ(k+8,k+8));
% covariance of equity yield, bond yield
uceb(ii) = uncvarZ(k+4,k+8);
% correlation of equity yield, bond yield
ureb(ii) = uceb(ii)/use(ii)/usb(ii);

% calc some expected inflation-equity correlations
% volatility of the endogenous
Zstds = sqrt(diag(uncvarZ));
% correlation matrix of the endogenous
Zcors = uncvarZ ./ (Zstds*Zstds');

% some correlations of expected inflation, the subjective bias, and equity yield components
picors(:,:,ii) = Zcors([1 (k+10) (k+4)],[1 (k+10) (k+4)]);

% some more covariance decompositions
% of equity yield components and expected inflation
vec1 = [(k+1:k+2) (sZ-1) (sZ-0)];
sum1 = k+4;
vec2 = 1;
sum2 = 1;
[p1,p2,p3] = varpctvar(uncvarZ,vec1,sum1,vec2,sum2);
% contribution of equity yield components to covaraince
% between expected inflation and the equity yield 
ey_epi(:,:,ii) = p3';

% throw out bad (nonstationary) draws
jj = 0;
ii = ii + 1;
if max(abs(eig(AY))) > 0.9950 | min(abs(eig(CovY))) <= 0 | min(abs(eig(uncvarZ))) <= 0,
    sprintf('warning: repeat draw due to (near-)nonstationaity')
    ii  = ii-1;
    jj  = 1;
end;

end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END BOOTSTRAPPING LOOP %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% report final statistics
% and bootstrapped confidence 
% intervals

% equity yield variance
use1 = use(1)
usec = prctile(use,[5 95])
% bond yield variance
usb1 = usb(1)
usbc = prctile(usb,[5 95])
% equity yield-bond yield covariance
uceb1 = uceb(1)
ucebc = prctile(uceb,[5 95])
% equity yield-bond yield correlation
ureb1 = ureb(1)
urebc = prctile(ureb,[5 95])

% equity yield variance component contributions
pe1 = pe1b(:,:,1);
pe1c = prctile(pe1b,[5 95],3);
ve1 = [pe1 reshape(pe1c,3,2)]
% bond yield variance components contributions
pb1 = pb1b(:,:,1);
pb1c = prctile(pb1b,[5 95],3);
vb1 = [pb1 reshape(pb1c,3,2)]
% equity yield-bond yield covariance component contributions
pc1 = pc1b(:,:,1);
pc1c = prctile(pc1b,[5 95],3);
cv1 = [pc1 reshape(pc1c,3,6)]

% equity yield variance component contributions
% with equity yield split up into spanned and unspanned parts
pe2 = pe2b(:,:,1);
pe2c = prctile(pe2b,[5 95],3);
ve2 = [pe2 reshape(pe2c,4,2)]
% bond yield component contributions
pb2 = pb2b(:,:,1);
pb2c = prctile(pb2b,[5 95],3);
vb2 = [pb2 reshape(pb2c,3,2)]
% equity yield-bond yield covariance component contributions
% with equity yield split up into spanned and unspanned parts
pc2 = pc2b(:,:,1);
pc2c = prctile(pc2b,[5 95],3);
cv2 = [pc2 reshape(pc2c,4,6)]

% correlations of equity yield components with expected inflation
picor1 = picors(:,:,1)
picorc = squeeze(prctile(picors,[5 95],3))
% correlations of equity yield components with expected inflation
% with equity yield split up into spanned and unspanned parts
ey_epi1 = ey_epi(:,:,1)
ey_epic = squeeze(prctile(ey_epi,[5 95],3))'


% Yt stats
% stds
% samp_std_W = std(W(2:end,:));
 StdY =  StdYb(:,:,1)'
 StdY_5_95 = squeeze(prctile(StdYb,[5 95],3))'
 StdY_c90w = StdY_5_95(2,:)-StdY_5_95(1,:)
 % sample autocorrelation of W
% cacW = corrcoef([W(2:end,:) W(1:end-1,:)]);
% samp_ac_W = diag(cacW(1:9,10:end))';
 ACorrY =  diag(ACorrYb(:,:,1))'
 ACorrY_5_95 = prctile(ACorrYb,[5 95],3);
 ACorrY_5_95 = [diag(ACorrY_5_95(:,:,1))';diag(ACorrY_5_95(:,:,2))']
 ACorrY_c90w = ACorrY_5_95(2,:)-ACorrY_5_95(1,:)
% cross-correlation
CorrY =  CorrYb(:,:,1)'
CorrY_5_95 = prctile(CorrYb,[5 95],3);
CorrY_c90w = CorrY_5_95(:,:,2)-CorrY_5_95(:,:,1)


