function ltest = cumby_huizinga_test(xp,u,s,q);
%
%This proc performs a ltest on residuals as described
%in Cumby and Huizinga (1990).  It allows nonzero "q".
%This test for serial correlation is robust to heteroskedasticity and
%predetermined and endogenous regressors.
%The procedure does not work when instrumental variables techniques are used.
%Throughout I will use the notation of Cumby and Huizinga (1990).
%
%Input: xp: (Txk) matrix of regressors
%       u: vector of residuals
%       s: number of autocorrelations to be included in test
%       q: number of nonzero autocorrelations
%
%When the regressor is exogenous, i.e. one is interested in a l-test
%for the autocorrelations of a particular series, e.g the 3 month forward
%bias with monthly data, simply put in a vector of ones for instance for xp
%and the time series itself as u.

tend = size(xp,1);
k    = size(xp,2);

% the mean of u will usually be zero (when it is a residual)
% but in the case of l-tests for a regular time series a
%     mean has to be taken out

u    = u - mean(u);
sigma = mean(u.^2); 

% construct Q(=qq here),B,D-matrices 
% temp is needed underneath 

ii = 1 + q;
qq = zeros(tend,s);
temp = zeros(tend,s);

while ii <= s+q,
    
    qq(ii+1:tend,ii-q) = u(1:tend-ii,:);
    temp(ii+1:tend,ii-q) = u(1:tend-ii,:);
    
    ii = ii + 1;
    
end;

if k == 1, % with exogenous regressors. b=0
    b = zeros(s,k);
else,
    b = (-qq'*xp)./(tend*sigma);    
end;

xx = inv(xp'*xp);
d = tend*xx; % in the present case, x=z

phi = zeros(2*s,s+k);
phi(1:s,1:k) = b*d;
phi(s+1:2*s,k+1:k+s) = eye(s)./sigma;

% compute PSI

w = xp.*repmat(u,1,k);

% as temp is padded with zeros at the start, the first q elements of
%      u will not be used in the cross-multiplication with temp

eta = [w repmat(u(1:tend,:),1,s).*temp];
psi = 1/tend*eta'*eta;

% when q>0, the serial correlation has to be taken into account 
% in computing the weighting matrix.  Cumby and Huizinga do report 
% that the Newey-West weighting scheme performs very poorly 
% in the Monte Carlos they carry out 

j = 1;
while j <= q,       
    etaj = eta(j+1:size(eta,1),:)'*eta(1:size(eta,1)-j,:);
    psi  = psi + (1/tend)*(1-j/(q+1))*(etaj' + etaj);    
    j = j + 1;
end;

% compute s auto correlations 
%    do small sample correction

j = 1 + q;
aust = zeros(s,1);
while j <= s+q,
    x = u(j+1:tend,:);
    xj = u(1:tend-j,:);
    aust(j-q,:) = sqrt((tend+2)/(tend-j))*(sum(x.*xj)/tend)./sigma;
    j = j + 1;
end;

% do chi square test 

vsq = phi*psi*phi';
v11 = vsq(1:s,1:s);
v21 = vsq(s+1:2*s,1:s);
v12 = vsq(1:s,s+1:2*s);
v22 = vsq(s+1:2*s,s+1:2*s);

v = v11 + v12 + v21 + v22;
chist = tend*aust'*inv(v)*aust;

% p-value

sprintf('Test Statistic');
chist;
sprintf('Chi-Square p-val');
pval = 1-cdf('chi2',chist,s);

ltest = [chist pval];






    
    
   
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    